@@ -2,7 +2,7 @@
 //
 // Livewire GPIO Server
 //
-// (C) Copyright 2014 Fred Gleason <fredg@paravelsystems.com>
+// (C) Copyright 2014-2019 Fred Gleason <fredg@paravelsystems.com>
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of version 2.1 of the GNU Lesser General Public
@@ -180,6 +180,7 @@ SyGpioServer::SyGpioServer(SyRouting *r,QObject *parent)
 {
   gpio_routing=r;
   gpio_eth_monitor=NULL;
+  gpio_serial=2;
 
   //
   // GPI Socket
@@ -432,8 +433,16 @@ void SyGpioServer::sendGpo(int srcnum,int line,bool state,bool pulse)
 }
 
 
-void SyGpioServer::sendGpo(int srcnum,const QString &code)
+void SyGpioServer::sendGpo(int srcnum,const QString &code,bool pulse)
 {
+  for(int i=0;i<code.length();i++) {
+    if(code.at(i)==QChar('L')) {
+      sendGpo(srcnum,i,true,pulse);
+    }
+    if(code.at(i)==QChar('H')) {
+      sendGpo(srcnum,i,false,pulse);
+    }
+  }
 }
 
 
@@ -464,47 +473,44 @@ void SyGpioServer::gpiReadyReadData()
   QString code;
 
   while((n=gpio_gpi_socket->readDatagram(data,1500,&addr,&port))>0) {
-    if(addr!=gpio_routing->nicAddress()) {
-      serial=((0xFF&data[4])<<24)+((0xFF&data[5])<<16)+((0xFF&data[6])<<8)+
-	(0xFF&data[7]);
-      if((gpio_src_addr_serials[addr.toIPv4Address()]!=serial)&&
-	 (gpio_src_addr_serials[addr.toIPv4Address()]!=(serial-1))) {
-	gpio_src_addr_serials[addr.toIPv4Address()]=serial;
-	uint16_t count=((0xFF&data[20])<<8)+(0xFF&data[21]);
-	srcnum=((0xFF&data[22+1])<<8)+(0xFF&data[22+2]);
-	code=gpio_gpi_codes.value(srcnum,"hhhhh");
-	for(uint16_t i=0;i<count;i++) {
-	  unsigned offset=22+i*6;
-	  //
-	  // FIXME: Is there actually a 'pulse' component for a GPI event?
-	  //        We assume not here (even though there is a place for one
-	  //        in the Switchyard API).
-	  //
-	  //	srcnum=((0xFF&data[offset+1])<<8)+(0xFF&data[offset+2]);
-	  int line=0x0D-(0xff&data[offset+3]);
-	  bool state=(data[offset+5]&0x01)!=0;
-	  //	printf("i: %d  srcnum: %d  line: %d  state: %d\n",
-	  //	       i,srcnum,line,state);
-	  if(state) {
-	    code.replace(line,1,"L");
-	  }
-	  else {
-	    code.replace(line,1,"H");
-	  }
-	  e=new SyGpioEvent(SyGpioEvent::TypeGpi,addr,port,srcnum,line,state,
-			    false);
-	  emit gpioReceived(e);
-	  emit gpiReceived(e->sourceNumber(),e->line(),e->state(),e->isPulse());
-	  gpio_routing->
-	    setGpi(e->sourceNumber(),e->line(),e->state(),e->isPulse());
-	  delete e;
+    serial=((0xFF&data[4])<<24)+((0xFF&data[5])<<16)+((0xFF&data[6])<<8)+
+      (0xFF&data[7]);
+    if((gpio_src_addr_serials[addr.toIPv4Address()]!=serial)&&
+       (gpio_src_addr_serials[addr.toIPv4Address()]!=(serial-1))) {
+      gpio_src_addr_serials[addr.toIPv4Address()]=serial;
+      uint16_t count=((0xFF&data[20])<<8)+(0xFF&data[21]);
+      srcnum=((0xFF&data[22+1])<<8)+(0xFF&data[22+2]);
+      code=gpio_gpi_codes.value(srcnum,"hhhhh");
+      for(uint16_t i=0;i<count;i++) {
+	unsigned offset=22+i*6;
+	//
+	// FIXME: Is there actually a 'pulse' component for a GPI event?
+	//        We assume not here (even though there is a place for one
+	//        in the Switchyard API).
+	//
+	int line=0x0D-(0xff&data[offset+3]);
+	bool state=(data[offset+5]&0x01)!=0;
+	//	printf("i: %d  srcnum: %d  line: %d  state: %d\n",
+	//	       i,srcnum,line,state);
+	if(state) {
+	  code.replace(line,1,"L");
 	}
-	gpio_gpi_codes[srcnum]=code.toLower();
-	eb=
-	  new SyGpioBundleEvent(SyGpioBundleEvent::TypeGpi,addr,port,srcnum,code);
-	emit gpioReceived(eb);
-	delete eb;
+	else {
+	  code.replace(line,1,"H");
+	}
+	e=new SyGpioEvent(SyGpioEvent::TypeGpi,addr,port,srcnum,line,state,
+			  false);
+	emit gpioReceived(e);
+	emit gpiReceived(e->sourceNumber(),e->line(),e->state(),e->isPulse());
+	gpio_routing->
+	  setGpi(e->sourceNumber(),e->line(),e->state(),e->isPulse());
+	delete e;
       }
+      gpio_gpi_codes[srcnum]=code.toLower();
+      eb=
+	new SyGpioBundleEvent(SyGpioBundleEvent::TypeGpi,addr,port,srcnum,code);
+      emit gpioReceived(eb);
+      delete eb;
     }
   }
 }
