@@ -60,6 +60,44 @@ SyLwrpServer::SyLwrpServer(SyRouting *routing)
 }
 
 
+void SyLwrpServer::sendGpiState(int slot,const QString &code)
+{
+  SyLwrpClientConnection *conn=NULL;
+
+  for(int i=0;i<SWITCHYARD_GPIO_BUNDLE_SIZE;i++) {
+    bool state=code.mid(i,1).toLower()=="l";
+    ctrl_routing->setGpiBySlot(slot,i,state,false);
+  }
+
+  for(unsigned i=0;i<ctrl_client_connections.size();i++) {
+    if((conn=ctrl_client_connections.at(i))!=NULL) {
+      if(conn->gpiAdded(slot)) {
+	SendCommand(i,QString().sprintf("GPI %d ",slot+1)+code);
+      }
+    }
+  }
+}
+
+
+void SyLwrpServer::sendGpoState(int slot,const QString &code)
+{
+  SyLwrpClientConnection *conn=NULL;
+
+  for(int i=0;i<SWITCHYARD_GPIO_BUNDLE_SIZE;i++) {
+    bool state=code.mid(i,1).toLower()=="l";
+    ctrl_routing->setGpoBySlot(slot,i,state,false);
+  }
+
+  for(unsigned i=0;i<ctrl_client_connections.size();i++) {
+    if((conn=ctrl_client_connections.at(i))!=NULL) {
+      if(conn->gpoAdded(slot)) {
+	SendCommand(i,QString().sprintf("GPO %d ",slot+1)+code);
+      }
+    }
+  }
+}
+
+
 void SyLwrpServer::newConnectionData()
 {
   unsigned id=0;
@@ -382,6 +420,211 @@ bool SyLwrpServer::ExecuteIfc(int ch,QStringList &args)
 }
 
 
+bool SyLwrpServer::ExecuteCfg(int ch,QStringList &args)
+{
+  unsigned slot=0;
+  unsigned srcnum=0;
+  QStringList fields;
+  QString addr="0.0.0.0";
+  bool save=false;
+  bool ok=false;
+
+  if(args.size()<2) {
+    return false;
+  }
+  if(args[1]!="GPO") {
+    return false;
+  }
+  if(args.size()==2) {
+    SendCommand(ch,"BEGIN");
+    for(unsigned i=0;i<ctrl_routing->gpos();i++) {
+      SendCommand(ch,CfgLine(i));
+    }
+    SendCommand(ch,"END");
+    return true;
+  }
+  if(args.size()>=3) {
+    slot=args[2].toInt(&ok)-1;
+    if((!ok)||(slot>=ctrl_routing->gpos())) {
+      return false;
+    }
+  }
+  if(args.size()==3) {
+    SendCommand(ch,CfgLine(slot));
+    return true;
+  }
+  for(int i=3;i<args.size();i++) {
+    fields=ParseField(SyAString(args[i]));
+    if(fields[0]=="NAME") {
+      if(fields.size()>1) {
+	ctrl_routing->setGpoName(slot,fields[1]);
+	save=true;
+      }
+    }
+    if(fields[0]=="SRCA") {
+      if(fields.size()>1) {
+	srcnum=fields[1].toUInt(&ok);
+	if(ok&&(srcnum<32768)) {
+	  ctrl_routing->setGpoAddress(slot,
+		       SyRouting::streamAddress(SyRouting::Stereo,srcnum));
+	  ctrl_routing->setGpoMode(slot,SyRouting::GpoFollowSource);
+	  save=true;
+	}
+	else {
+	  QHostAddress addr(fields[1]);
+	  if(!addr.isNull()) {
+	    ctrl_routing->setGpoAddress(slot,addr);
+	    ctrl_routing->setGpoMode(slot,SyRouting::GpoFollowSource);
+	    save=true;
+	  }
+	  else {
+	    QStringList f0=fields[1].split("/");
+	    if(f0.size()==2) {
+	      addr.setAddress(f0[0]);
+	      int snake_slot=f0[1].toInt(&ok)-1;
+	      if((!addr.isNull())&&ok) {
+		ctrl_routing->setGpoMode(slot,SyRouting::GpoSnake);
+		ctrl_routing->setGpoAddress(slot,addr);
+		ctrl_routing->setGpoSnakeSlot(slot,snake_slot);
+		save=true;
+	      }
+	      else {
+		return false;
+	      }
+	    }
+	    else {
+	      ctrl_routing->setGpoMode(slot,SyRouting::GpoFollowSource);
+	      ctrl_routing->setGpoAddress(slot,QHostAddress());
+	      ctrl_routing->setGpoSnakeSlot(slot,0);
+	      save=true;
+	    }
+	  }
+	}
+      }
+      else {
+	ctrl_routing->setGpoAddress(slot,"0.0.0.0");
+      }
+    }
+  }
+
+  if(save) {
+    ctrl_routing->save();
+    emit gpoCfgChanged(slot,ctrl_routing->gpoMode(slot),
+		       ctrl_routing->gpoAddress(slot),
+    		       ctrl_routing->gpoSnakeSlot(slot));
+  }
+  SendCommand(ch,CfgLine(slot));
+  
+  return true;
+}
+
+
+bool SyLwrpServer::ExecuteAdd(int ch,QStringList &args)
+{
+  unsigned slot;
+  bool ok=false;
+  bool ret=false;
+
+  /*
+  if(args.size()!=2) {
+    return ret;
+  }
+  */
+
+  if(args[1]=="GPI") {
+    if(args.size()==2) {
+      for(unsigned i=0;i<ctrl_routing->gpis();i++) {
+	ctrl_client_connections[ch]->gpiAdd(i);
+      }
+      SendCommand(ch,"BEGIN");
+      for(unsigned i=0;i<ctrl_routing->gpis();i++) {
+	SendCommand(ch,GpiLine(i));
+      }
+      SendCommand(ch,"END");
+      ret=true;
+    }
+    if(args.size()==3) {
+      slot=args[2].toUInt(&ok)-1;
+      if(ok&&(slot<ctrl_routing->gpis())) {
+	ctrl_client_connections[ch]->gpiAdd(slot);
+	SendCommand(ch,GpiLine(slot));
+	ret=true;
+      }
+    }
+  }
+  if(args[1]=="GPO") {
+    if(args.size()==2) {
+      for(unsigned i=0;i<ctrl_routing->gpos();i++) {
+	ctrl_client_connections[ch]->gpoAdd(i);
+      }
+      SendCommand(ch,"BEGIN");
+      for(unsigned i=0;i<ctrl_routing->gpos();i++) {
+	SendCommand(ch,GpoLine(i));
+      }
+      SendCommand(ch,"END");
+      ret=true;
+    }
+    if(args.size()==3) {
+      slot=args[2].toUInt(&ok)-1;
+      if(ok&&(slot<ctrl_routing->gpos())) {
+	ctrl_client_connections[ch]->gpoAdd(slot);
+	SendCommand(ch,GpoLine(slot));
+	ret=true;
+      }
+    }
+  }
+
+  return ret;
+}
+
+
+bool SyLwrpServer::ExecuteDel(int ch,QStringList &args)
+{
+  unsigned slot;
+  bool ok=false;
+  bool ret=false;
+
+  /*
+  if(args.size()!=2) {
+    return ret;
+  }
+  */
+
+  if(args[1]=="GPI") {
+    if(args.size()==2) {
+      for(unsigned i=0;i<ctrl_routing->gpis();i++) {
+	ctrl_client_connections[ch]->gpiDel(i);
+      }
+      ret=true;
+    }
+    if(args.size()==3) {
+      slot=args[2].toUInt(&ok)-1;
+      if(ok&&(slot<ctrl_routing->gpis())) {
+	ctrl_client_connections[ch]->gpiDel(slot);
+	ret=true;
+      }
+    }
+  }
+  if(args[1]=="GPO") {
+    if(args.size()==2) {
+      for(unsigned i=0;i<ctrl_routing->gpos();i++) {
+	ctrl_client_connections[ch]->gpoDel(i);
+      }
+      ret=true;
+    }
+    if(args.size()==3) {
+      slot=args[2].toUInt(&ok)-1;
+      if(ok&&(slot<ctrl_routing->gpos())) {
+	ctrl_client_connections[ch]->gpoDel(slot);
+	ret=true;
+      }
+    }
+  }
+
+  return ret;
+}
+
+
 QString SyLwrpServer::SrcLine(int slot)
 {
   return QString().
@@ -437,6 +680,34 @@ QString SyLwrpServer::GpoLine(int slot)
 }
 
 
+QString SyLwrpServer::CfgLine(int slot)
+{
+  QString ret=QString().sprintf("CFG GPO %d",slot+1);
+
+  ret+=" NAME:\""+ctrl_routing->gpoName(slot)+"\"";
+  bool srca_sent=false;
+  if(ctrl_routing->gpoMode(slot)==SyRouting::GpoFollowSource) {
+    if(SyRouting::livewireNumber(ctrl_routing->gpoAddress(slot))!=0) {
+      ret+=QString().sprintf(" SRCA:\"%u\"",
+			     SyRouting::livewireNumber(ctrl_routing->gpoAddress(slot)));
+      srca_sent=true;
+    }
+  }
+  if(ctrl_routing->gpoMode(slot)==SyRouting::GpoSnake) {
+    if(!ctrl_routing->gpoAddress(slot).isNull()) {
+      ret+=QString(" SRCA:")+ctrl_routing->gpoAddress(slot).toString()+
+	QString().sprintf("/%d",ctrl_routing->gpoSnakeSlot(slot)+1);
+      srca_sent=true;
+    }
+  }
+  if(!srca_sent) {
+    ret+=" SRCA:";
+  }
+
+  return ret;
+}
+
+
 QStringList SyLwrpServer::ParseField(const SyAString &field)
 {
   QStringList ret=field.split(":");
@@ -491,6 +762,15 @@ void SyLwrpServer::ParseCommand(int id)
   if(args[0]=="ifc") {
     ok=ExecuteIfc(id,args);
   }
+  if(args[0]=="cfg") {
+    ok=ExecuteCfg(id,args);
+  }
+  if(args[0]=="add") {
+    ok=ExecuteAdd(id,args);
+  }
+  if(args[0]=="del") {
+    ok=ExecuteDel(id,args);
+  }
   if(!ok) {
     SendCommand(id,"ERROR 1000 bad command");
   }
