@@ -2,7 +2,7 @@
 //
 // AoIP stream routing configuration
 //
-// (C) 2014 Fred Gleason <fredg@paravelsystems.com>
+// (C) 2014-2018 Fred Gleason <fredg@paravelsystems.com>
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of version 2.1 of the GNU Lesser General Public
@@ -35,6 +35,7 @@
 class SyRouting
 {
  public:
+  enum GpoMode {GpoFollowSource=0,GpoSnake=1};
   enum Realm {Stereo=0,Backfeed=1,Surround=2};
   SyRouting(unsigned d_slots,unsigned s_slots,
 	    unsigned gpis=0,unsigned gpos=0);
@@ -65,12 +66,21 @@ class SyRouting
   QString dstName(int slot) const;
   void setDstName(int slot,const QString &str);
   int dstMeterLevel(int slot,int chan) const;
-  bool gpiState(int gpi,int line) const;
   bool gpiStateBySlot(int slot,int line) const;
-  void setGpi(int gpi,int line,bool state,bool pulse);
-  bool gpoState(int gpo,int line) const;
+  void setGpi(int srcnum,int line,bool state,bool pulse);
+  void setGpiBySlot(int slot,int line,bool state,bool pulse);
   bool gpoStateBySlot(int slot,int line) const;
-  void setGpo(int gpo,int line,bool state,bool pulse);
+  void setGpo(int srcnum,int line,bool state,bool pulse);
+  void setGpoBySlot(int slot,int line,bool state,bool pulse);
+  GpoMode gpoMode(int slot) const;
+  void setGpoMode(int slot,GpoMode mode);
+  QHostAddress gpoAddress(int slot) const;
+  void setGpoAddress(int slot,const QHostAddress &addr);
+  void setGpoAddress(int slot,const QString &addr);
+  int gpoSnakeSlot(int slot) const;
+  void setGpoSnakeSlot(int slot,int snake_slot);
+  QString gpoName(int slot) const;
+  void setGpoName(int slot,const QString &str);
   unsigned nicQuantity() const;
   QHostAddress nicAddress(unsigned n);
   QHostAddress nicNetmask(unsigned n);
@@ -91,6 +101,7 @@ class SyRouting
   float src_meter[SWITCHYARD_MAX_SLOTS][SWITCHYARD_MAX_CHANNELS];
   uint32_t dst_addr[SWITCHYARD_MAX_SLOTS];
   float dst_meter[SWITCHYARD_MAX_SLOTS][SWITCHYARD_MAX_CHANNELS];
+  uint32_t gpo_addr[SWITCHYARD_MAX_SLOTS];
   void writeRtpData(unsigned src_slot,const char *data,int len) const;
   static unsigned livewireNumber(const QHostAddress &addr);
   static QHostAddress streamAddress(Realm realm,uint16_t lw_num);
@@ -99,10 +110,12 @@ class SyRouting
   static QString socketErrorString(const QString &msg);
 
  private:
-  int GetSlotByGpio(int gpio) const;
   void LoadInterfaces();
   QString sy_src_names[SWITCHYARD_MAX_SLOTS];
   QString sy_dst_names[SWITCHYARD_MAX_SLOTS];
+  QString sy_gpo_names[SWITCHYARD_MAX_SLOTS];
+  GpoMode sy_gpo_modes[SWITCHYARD_MAX_SLOTS];
+  int sy_gpo_snake_slots[SWITCHYARD_MAX_SLOTS];
   std::vector<bool> sy_gpi_states;
   std::vector<bool> sy_gpo_states;
   std::vector<QHostAddress> sy_nic_addresses;
